"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderOperation = void 0;
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
const lodash_1 = __importStar(require("lodash"));
async function renderOperation(operation) {
    const { method, url } = operation.__extra__;
    const requestLine = `${method.toUpperCase()} ${url}`;
    const op = preprocessOperation(operation);
    const omitAxiosKeysType = [
        'method',
        'url',
        op.params.query ? 'params' : null,
        op.data ? 'data' : null,
    ]
        .filter(Boolean)
        .map((el) => JSON.stringify(el))
        .join('|');
    const configType = [
        `Omit<AxiosRequestConfig<any>, ${omitAxiosKeysType}>`,
        op.params.query
            ? op.params.query.required
                ? `{ params: __BaseTypes__['Params'] }`
                : `{ params?: __BaseTypes__['Params'] }`
            : '',
        op.data
            ? op.data.required
                ? "{ data: __BaseTypes__['Body'] }"
                : "{ data?: __BaseTypes__['Body'] }"
            : '',
    ]
        .filter(Boolean)
        .join('&');
    const pathParams = op.params.path?.value.map((el) => ({
        ...el,
        name: (0, lodash_1.camelCase)(el.name),
    }));
    const simplePathArgs = op.params.path?.isSimple
        ? pathParams
            ?.map(({ required, schema, name }) => {
            const isI64 = schema?.format === 'int64';
            const realType = schema?.type !== 'string' && !isI64 ? 'number' : 'string';
            return `${isI64 ? '/** @format int64 */' : ''} ${name}: ${realType}${!required ? '| undefined' : ''}`;
        })
            .join(',')
        : '';
    const fnArgs = [
        op.params.path?.isSimple
            ? simplePathArgs
            : op.params.path
                ? `pathParams: __BaseTypes__["PathParams"]`
                : '',
        op.params.query?.required || op.data?.required
            ? `__options: __Config`
            : `__options?: __Config`,
    ]
        .filter(Boolean)
        .join(',');
    const pathExpr = op.params.path?.isSimple
        ? `interpolatePath(__url, {${pathParams?.map((el) => el.name).join(',') || ''}})`
        : op.params.path
            ? `interpolatePath(__url, pathParams)`
            : '__url';
    const fnResult = lodash_1.default.trim(`
     (${fnArgs}) => ({
      ...__options,
      url: ${pathExpr},
      method: "${method}",
      data: ${op.data?.isMultipart ? 'toFormData' : ''}(__options?.data)
    })
  `);
    const interfaceCode = (await compileToTs(op.helperSchema, '__BaseTypes__'))
        .replace(/([\s])export[\s]+(type|enum|interface)/g, '$1$2')
        .replace(/\/\*\*\s+\*\s*([^\n]+)\s+\*\//g, '/** $1 */');
    return `
    /**
     * @summary ${requestLine}
     * @path ${operation.summary}
     */
    function ${operation.operationId}() {
      const __path = "${requestLine}" as const;
      const __url = __path.slice(${method.length + 1});

      ${interfaceCode}

      type __Config = ${configType};
      const __request = ${fnResult};

      return {
        [__path]: [
          0 as unknown as __BaseTypes__ & { ResponseData:  PickData<__BaseTypes__["Response"]> },
          __request
        ] as const
      };
    }
  `;
}
exports.renderOperation = renderOperation;
function compileToTs(schema, name) {
    return (0, json_schema_to_typescript_1.compile)(schema, name, {
        format: false,
        bannerComment: undefined,
        additionalProperties: false,
        declareExternallyReferenced: true,
        unreachableDefinitions: false,
    });
}
function preprocessOperation(operation) {
    const { requestBody, isMultipart, responseData } = operation;
    const pathParams = operation.parameters.pathParams;
    const queryParams = operation.parameters.queryParams;
    const hasPathParams = pathParams.length > 0;
    const hasQuery = queryParams.length > 0;
    const hasRequestBody = Boolean(requestBody?.schema);
    const hasRequiredQuery = hasQuery && queryParams.some((el) => el.required);
    const hasRequiredBody = requestBody?.schema?.required
        ?.length;
    const isSimplePathParams = pathParams.length > 0 &&
        pathParams.length < 3 &&
        pathParams.every((el) => el.required);
    const helperSchema = {
        type: 'object',
        summary: operation.summary,
        deprecated: operation.deprecated,
        description: operation.description || operation.summary,
        required: ['Body', 'Response', 'Params', 'PathParams'],
        properties: {
            Body: hasRequestBody
                ? processSchemaObject(requestBody.schema, 'Body')
                : undefined,
            Response: responseData?.schema
                ? processSchemaObject(responseData.schema, 'Response')
                : undefined,
            Params: queryParams.length
                ? {
                    type: 'object',
                    properties: (0, lodash_1.fromPairs)(queryParams.map((el) => [
                        el.name,
                        processSchemaObject(el.schema, 'Params'),
                    ])),
                }
                : undefined,
            PathParams: queryParams.length
                ? {
                    type: 'object',
                    properties: (0, lodash_1.fromPairs)(pathParams.map((el) => [
                        el.name,
                        processSchemaObject(el.schema, 'PathParams'),
                    ])),
                }
                : undefined,
        },
    };
    return {
        params: {
            path: hasPathParams
                ? {
                    value: pathParams,
                    isSimple: isSimplePathParams,
                }
                : null,
            query: hasQuery
                ? {
                    value: queryParams,
                    required: hasRequiredQuery,
                }
                : null,
        },
        data: requestBody
            ? {
                value: requestBody,
                isMultipart,
                required: hasRequiredBody,
            }
            : null,
        helperSchema,
    };
}
function processSchemaObject(_schema, interfaceName) {
    if (!_schema)
        return;
    const schema = _schema;
    const { type, format } = schema;
    const title = schema.title && (interfaceName || schema.title);
    if (type === 'array') {
        return {
            ...schema,
            title,
            items: schema.items && processSchemaObject(schema.items, `${interfaceName}Item$`),
        };
    }
    if (type === 'object') {
        return {
            ...schema,
            title,
            properties: (0, lodash_1.mapValues)(schema.properties, (val, key) => processSchemaObject(val, `${interfaceName}_${lodash_1.default.upperFirst(key)}`)),
        };
    }
    // int64 处理
    const type2 = type === 'integer' && format === 'int64' ? 'string' : type;
    return {
        ...schema,
        type: type2,
        title,
        description: [schema.description || '', format ? `@format ${format}` : '']
            .join('\n')
            .trim(),
    };
}
//# sourceMappingURL=render.js.map